<?php
/**
 * @file
 * classes for entity object, not depending on LS.
 */

/**
 * Class managing building dynamic entities.
 *
 * Building entity_info, entity_property_info, schema. Take care of Drupal
 * fields and custom fields.
 */
class LimesurveySyncDpInfos {
  // store the class info name into entity_info().
  // Default label.
  protected $label = 'Entity';
  // Default plural label.
  protected $plural_label = 'Entities';
  // Entity machine name : no spaces.
  protected $entityname = 'entityname';
  protected $module = 'esan';

  // Add custom keys to the entity info datas :
  // And the prefix root path for view, edit, ...:
  public $crud_path = '';
  // Create an overview page on the path.
  protected $overview_path;


  // Uri for the overview page.
  public $root_admin_ui = '';
  // If the entity manage revision.
  protected $revision = TRUE;
  protected $entity_keys = array();

  // If the entity contains 'created' and 'changed' dates fields.
  protected $date_col = TRUE;
  // If the entity contains title field.
  protected $title_col = TRUE;
  // If the entity contains 'uid' user field.
  protected $user_col = TRUE;
  // If the entity contains 'language' language field.
  protected $language_col = TRUE;
  // If the entity can be published/unpublished.
  protected $status_col = TRUE;
  // If the comment are open or not.
  protected $comment_col = TRUE;
  // If the entity can be promoted to front page.
  protected $promote_col = TRUE;
  // If the entity can be set as sticky.
  protected $sticky_col = TRUE;
  // colomns for bundles.
  protected $type_col_entity = FALSE;
  protected $type_col_bundle = FALSE;
  protected $label_col = FALSE;
  protected $weight_col = FALSE;
  protected $description_col = FALSE;
  protected $help_col = FALSE;

  // If the entity is fieldable.
  protected $fieldable = TRUE;
  // If the entity is exportable.
  protected $exportable = FALSE; //TRUE;

  // Correspond to entity_info['bundle of'].
  protected $bundle_of = FALSE;
//  Correspond to the bundle entity type.
  protected $has_bundle = FALSE;
  // Bundle wildcard.
  protected $bundle_wildcard = NULL;
  // Retreiving the entity info of the bundle during declaring entity_info()
  // create an infinite loop. Cache doesn't help. As a workarround, assume that
  // the bundle table name is the bundle entity name or set it here.
  protected $menu_wildcard = NULL;
  protected $bundle_tablename = FALSE;

  // the primary key.
  protected $primary_key = '';
  protected $tablename = '';
  protected $tablename_revision = '';
  protected $entityclassname = '';
  protected $controller_class = '';
  protected $ui_controller_class = '';
  protected $entity_access_callback = '';
  protected $entity_label_callback = '';

  // Permissions
  protected $perm_add = TRUE;
  protected $perm_update = TRUE;
  protected $perm_view = TRUE;
  protected $perm_delete = TRUE;

  // wrappers
  protected $views_controller_class = 'EntityDefaultViewsController';
  protected $rules_controller_class = 'EntityDefaultRulesController';
  // @TODO : extend EntityDefaultExtraFieldsController for displaying properties.
  protected $metadata_controller_class = FALSE; //'EntityDefaultMetadataController';

  /**
   * Set the new object.
   */
  public function __construct() {
    if (empty($this->tablename)) {
      $this->tablename = $this->entityname;
    }
    if (empty($this->tablename_revision)) {
      $this->tablename_revision = $this->entityname . '_revision';
    }
    if (empty($this->primary_key)) {
      $this->primary_key = 'nid';
    }
    if (empty($this->entityclassname)) {
      $this->entityclassname = 'Entity';
    }
    if (empty($this->controller_class)) {
      $this->controller_class = 'EntityAPIControllerExportable';
    }
    if (empty($this->ui_controller_class)) {
      $this->ui_controller_class = 'EntityDefaultUIController';
    }
    if (empty($this->entity_access_callback)) {
      $this->entity_access_callback = 'limesurvey_sync_api_entity_entity_access';
    }
    if (!empty($this->exportable)) {
      $this->entity_keys['status'] = 'export_status';
      $this->entity_keys['module'] = 'export_module';
    }

    $empty_crup_path = FALSE;
    if (empty($this->crud_path)) {
      $empty_crup_path = TRUE;
      $this->crud_path = $this->entityname;
    }
    // Automatic config for bundles and associated entity.
    if ($this->bundle_of) {
      // This is the bundle entity.
      $this->type_col_bundle = TRUE;
      $this->type_col_entity = FALSE;
      $this->label_col = TRUE;
      $this->title_col = FALSE;
      $this->description_col = TRUE;
      $this->help_col = TRUE;
      // Content type has no view page, only edit page.
      //$this->perm_view = FALSE;
      // For Bundle : crud into admin page.
      // Set myentity/add, myentity/edit, etc... instead of
      // admin/structure/myentity/add, admin/structure/myentity/edit, etc...
      if (empty($this->root_admin_ui)) {
        $this->root_admin_ui = "admin/structure/{$this->crud_path}";
      }
      if ($empty_crup_path) {
        $this->crud_path = $this->root_admin_ui;
      }
      if (empty($this->overview_path)) {
        $this->overview_path = $this->root_admin_ui;
      }
    }

    if (!empty($this->has_bundle)) {
      // This is the entity that contain a bundle.
      $this->type_col_bundle = FALSE;
      $this->type_col_entity = TRUE;
      if (empty($this->bundle_wildcard)) {
        $this->bundle_wildcard = '%' . $this->has_bundle;
      }
      if (empty($this->bundle_tablename)) {
        // Retreiving the entity info of the bundle during declaring entity_info()
        // create an infinite loop. Cache doesn't help. As a workarround, assume that
        // the bundle table name is the bundle entity name.
        $this->bundle_tablename = $this->has_bundle;
      }
    }

    if (empty($this->root_admin_ui)) {
      $this->root_admin_ui = 'admin/structure/' . $this->crud_path;
    }
    if (empty($this->overview_path)) {
      $this->overview_path = 'admin/content/' . $this->crud_path;
    }
  }

  /**
   * Inform the system about one or more entity types
   *
   * Building entity_info corresponding to settings above.
   *
   * @see hook_entity_info()
   */
  public function entity_info() {

    $return = array();
    $entity_info = array();

    // Create the entity name :
    $entity_name = $this->entityname;
    $path_admin = "{$this->root_admin_ui}";

    $entity_info = array(
      'label' => $this->label,
      'plural label' => $this->plural_label,
      'entity class' => $this->entityclassname,
      'controller class' => $this->controller_class,
      'uri callback' => 'entity_class_uri',
      'module' => $this->module,
      'translation' => array(
        'locale' => TRUE,
      ),
      'base table' => $this->tablename,
    );
    if ($this->revision) {
      $entity_info['revision table'] = $this->tablename_revision;
    }

    // Build the 'entity keys' according to the setting of each entity.
    $entity_keys = $this->entity_keys;
    if (!empty($this->primary_key)) {
      $entity_keys['id'] = $this->primary_key;
    }
    if ($this->revision) {
      $entity_keys['revision'] = 'vid';
    }
    if ($this->language_col) {
      $entity_keys['language'] = 'language';
    }
    if ($this->type_col_bundle) {
      //$entity_keys['bundle'] = 'type';
      $entity_keys['name'] = 'type';
    }
    if ($this->type_col_entity) {
      $entity_keys['bundle'] = 'type';
    }
    if ($this->label_col || $this->title_col) {
      $entity_keys['label'] = ($this->title_col) ? 'title' : 'label';
    }
    $entity_info['entity keys'] = $entity_keys;

    // If fieldable.
    $entity_info['fieldable'] = $this->fieldable;
    $entity_info['exportable'] = $this->exportable;
    if (!empty($this->entity_access_callback)) {
      $entity_info['access callback'] = $this->entity_access_callback;
    }
    if ($this->bundle_of) {
      $entity_info['bundle of'] = $this->bundle_of;
    }

    // Add a custom entries : 'root crud path' for the root path for editing, viewing, etc...
    $entity_info['crud path'] = $this->crud_path;
    $entity_info['overview path'] = $this->overview_path;

    // Enable the entity API's admin UI.
    $entity_info['admin ui']['path'] = $path_admin;
    $entity_info['admin ui']['controller class'] = $this->ui_controller_class;
    if (!empty($this->menu_wildcard)) {
      $entity_info['admin ui']['menu wildcard'] = $this->menu_wildcard;
    }

    if (!empty($this->entity_label_callback)) {
      // Set dynamic titles.
      $entity_info['label callback'] = $this->entity_label_callback;
    }

    // Manage bundles.
    $entity_info['bundles'] = array();
    if ($this->has_bundle) {
      $entity_info['bundle keys'] = array('bundle' => 'type');
      // Retreiving the entity info of the bundle can create a infinite loop.
      // Cache doesn't help. As a workarround, assume that the bundle table name
      // is the bundle entity name.
      $bundle_table_name = $this->bundle_tablename;
      if (db_table_exists($bundle_table_name)) {
        $query = db_query('SELECT * FROM {' . $bundle_table_name . '}');
        while ($bundle_entity = $query->fetchObject()) {
          $type = $bundle_entity->type;
          $path_bundle_root = $this->root_admin_ui;
          $id_count = count(explode('/', $path_bundle_root));
          $entity_info['bundles'][$type] = array(
            'label' => $bundle_entity->label,
            'admin' => array(
              'path' => $path_bundle_root . '/manage/' . $this->bundle_wildcard,
              'real path' => $path_bundle_root . '/manage/' . str_replace('_', '-', $type),
              'bundle argument' => $id_count + 1,
              'access arguments' => array("administer {$this->has_bundle}"),
            ),
          );
        }
      }
    }
    elseif ($this->fieldable) {
      // Create 1 bundle : to allow to add fields to if fiedable is TRUE.
      $entity_info['bundles'][$entity_name] = array(
        'label' => 'Manage ' . $this->plural_label,
        'admin' => array(
          'path' => $path_admin,
          'access arguments' => array("administer {$this->entityname}"),
        ),
      );
    }

    $entity_info['view modes'] = array(
      // @todo we should support view modes.
      'full' => array(
        'label' => t('Full'),
        'custom settings' => FALSE,
      ),
    );

    // classes.
    if (!empty($this->views_controller_class)) {
      $entity_info['views controller class'] = $this->views_controller_class;
    }
    if (!empty($this->rules_controller_class)) {
      $entity_info['rules controller class'] = $this->rules_controller_class;
    }
    if (!empty($this->metadata_controller_class)) {
      $entity_info['metadata controller class'] = $this->metadata_controller_class;
    }

    // Set the array.
    $return[$entity_name] = $entity_info;
    return $return;
  }

  /**
   * Define the current version of the database schema (hook_schema()).
   *
   * Building schema corresponding to settings above.
   *
   * @return
   *   An array, the tables schema keyed by the table names.
   *
   *   @see hook_schema()
   */
  public function schema($datas = array()) {

    $schema = array();
    $entity_table = $this->tablename;
    $schema[$entity_table] = $this->schema_dp_table(FALSE, $datas);

    $schema[$entity_table]['fields'] = array_merge($this->schema_dp_fields(FALSE), $this->schema_custom_fields(FALSE, $datas));

    // Create the revision table.
    if ($this->revision) {
      $revision_table = $this->tablename_revision;
      $schema[$revision_table] = $this->schema_dp_table(TRUE, $datas);
      $schema[$revision_table]['fields'] = array_merge($this->schema_dp_fields(TRUE), $this->schema_custom_fields(TRUE, $datas));
    }

    return $schema;
  }

  /**
   * Provide schema table informations.
   *
   * Especially indexes, table description. Fields are defined outside from here.
   *
   * @return
   *   An array, the table infos keyed by properties ('description', etc...).
   *
   * @see LimesurveySyncDpInfos::schema()
   */
  protected function schema_dp_table($revision = FALSE, $datas = array()) {

    $schema = array(
      'description' => (!$revision) ? "Stores {$this->label} datas" : "Stores {$this->label} revisions datas",
    );

    if (!isset($schema['primary key'])) {
      $schema['primary key'] = array((!$revision) ? $this->primary_key : 'vid');
    }

    if ($revision) {
      $schema['unique keys']['vid'] = array('vid');
    }

    // Foreigner keys.
    if ($this->user_col) {
      $user_fk = (!$revision) ? 'entity_author' : 'version_author';
      $field_fk = (!$revision) ? 'uid' : 'revision_uid';
      $schema['foreign keys'][$user_fk] = array(
        'table' => 'users',
        'columns' => array($field_fk => 'uid'),
      );
    }
    if ($this->type_col_entity && !$revision) {
      $schema['foreign keys']['type'] = array(
        // Assume the entity name and table name are egal.
        'table' => $this->bundle_tablename,
        'columns' => array('type' => 'type'),
      );
    }

    if ($this->revision) {
      $rev_fk = (!$revision) ? 'entity_revision' : 'versioned_entity';
      $col_fk = (!$revision) ? 'vid' : $this->primary_key;
      $schema['foreign keys'][$rev_fk] = array(
        'table' => (!$revision) ? $this->tablename_revision : $this->tablename,
        'columns' => array($col_fk => $col_fk),
      );
    }

    // indexes.
    if ($this->type_col_entity && !$revision) {
      $schema['indexes']['type'] = array('type');
    }
    if ($this->user_col) {
      $schema['indexes']['uid'] = (!$revision) ? array('uid') : array('revision_uid');
    }
    if ($revision) {
      $schema['indexes'][$this->primary_key] = array($this->primary_key);
    }
    else {
      if ($this->date_col) {
        $schema['indexes']['entity_changed'] = array('changed');
        $schema['indexes']['entity_created'] = array('created');
      }
      if ($this->language_col) {
        $schema['indexes']['language'] = array('language');
      }
      if ($this->promote_col && $this->status_col && $this->sticky_col && $this->date_col) {
        $schema['indexes']['entity_frontpage'] = array('promote', 'status', 'sticky', 'created');
      }
      // TODO : add :
      // 'node_status_type'    => array('status', 'type', 'nid'),
      //'node_title_type'     => array('title', array('type', 4)),
      //'node_type'           => array(array('type', 4)),
      //'tnid'
    }

    return $schema;
  }

  /**
   * Define schema for DP fields.
   *
   * DP fields are 'uid', 'created', 'changed', 'vid', etc...
   *
   * @param $revision
   *   A Boolean, TRUE for revision table fields. Default to FALSE.
   *
   * @return
   *   An array, the DP schema fields keyed by there name.
   *
   * @see LimesurveySyncDpInfos::schema()
   */
  protected function schema_dp_fields($revision = FALSE) {
    // set date fields.
    // Fix primary key.
    $colomns = array();
    if (!$revision) {
      $colomns[$this->primary_key] = array(
        'description' => "Primary Key: Unique {$this->label} identifier.",
        'type' => 'serial',
        'unsigned' => TRUE,
        'not null' => TRUE,
      );
    }
    else {
      $colomns[$this->primary_key] = array(
        'description' => 'The {' . $this->tablename . '} this version belongs to.',
        'type' => 'int',
        'unsigned' => TRUE,
        'not null' => TRUE,
        'default' => 0,
      );
    }

    if ($this->revision) {
      if (!$revision) {
        $colomns['vid'] = array(
          'description' => 'The current {' . $this->tablename_revision . '}.vid version identifier.',
          'type' => 'int',
          'unsigned' => TRUE,
          'not null' => FALSE,
          'default' => NULL,
        );
      }
      else {
        $colomns['vid'] = array(
          'type' => 'serial',
          'unsigned' => TRUE,
          'not null' => TRUE,
          'description' => 'The primary identifier for this revision.',
        );
      }
    }

    if ($this->type_col_bundle) {
      $colomns['type'] = array(
        'description' => "Unique identifier, the machine-readable name of this {$this->label}.",
        'type' => 'varchar',
        'length' => 255,
        'not null' => TRUE,
      );
    }
    if ($this->type_col_entity) {
      // For node table, type is not stored into the revision table.
      if (!$revision) {
        $colomns['type'] = array(
          'description' => "The machine-readable name of this {$this->label}.",
          'type' => 'varchar',
          'length' => 255,
          'not null' => TRUE,
        );
      }
    }

    if ($this->label_col) {
      $colomns['label'] = array(
        'description' => "The human-readable name of this {$this->label}",
        'type' => 'varchar',
        'length' => 255,
        'not null' => TRUE,
        'default' => '',
      );
    }

    if ($this->description_col) {
      $colomns['description'] = array(
        'description' => "A brief description of this {$this->label}.",
        'type' => 'text',
        'not null' => TRUE,
        'size' => 'medium',
        'translatable' => TRUE,
      );
    }

    if ($this->help_col) {
      $colomns['help'] = array(
        'description' => "Help information shown to the user when creating a content of this {$this->label}.",
        'type' => 'text',
        'not null' => TRUE,
        'size' => 'medium',
        'translatable' => TRUE,
      );
    }
    // Set user field
    if ($this->user_col) {
      if (!$revision) {
        $colomns['uid'] = array(
          'description' => "The {users}.uid that owns this {$this->label}; initially, this is the user that created it.",
          'type' => 'int',
          'not null' => TRUE,
          'default' => 0,
        );
      }
      else {
        $colomns['revision_uid'] = array(
          'description' => 'The {users}.uid that created this version.',
          'type' => 'int',
          'not null' => TRUE,
          'default' => 0,
        );
      }
    }

    if ($this->date_col) {
      if (!$revision) {
        $colomns['created'] = array(
          'description' => "The Unix timestamp when the {$this->label} was created.",
          'type' => 'int',
          'not null' => TRUE,
          'default' => 0,
        );
        $colomns['changed'] = array(
          'description' => "The Unix timestamp when the {$this->label} was most recently saved.",
          'type' => 'int',
          'not null' => TRUE,
          'default' => 0,
        );
      }
      else {
        $colomns['timestamp'] = array(
          'description' => "The Unix timestamp when this version was created.",
          'type' => 'int',
          'not null' => TRUE,
          'default' => 0,
        );
      }
    }

    if ($this->language_col) {
      $colomns['language'] = array(
        'description' => "The Drupal {languages}.language of this {$this->label}.",
        'type' => 'varchar',
        'length' => 12,
        'not null' => TRUE,
        'default' => '',
      );
    }

    if ($this->title_col) {
      $colomns['title'] = array(
        'description' => (!$revision) ? "The title of this {$this->label}, always treated as non-markup plain text." : 'The title of this version.',
        'type' => 'varchar',
        'length' => 255,
        'not null' => TRUE,
        'default' => '',
      );
    }

    $rev_data = ($revision) ? ' (at the time of this revision)' : '';

    if ($this->weight_col) {
      $colomns['weight'] = array(
        'type' => 'int',
        'not null' => TRUE,
        'default' => 0,
        'size' => 'tiny',
        'description' => "The weight of this {$this->label} in relation to others.",
      );
    }

    if ($revision) {
      $colomns['log'] = array(
        'description' => 'The log entry explaining the changes in this version.',
        'type' => 'text',
        'not null' => TRUE,
        'size' => 'big',
      );
    }

    if ($this->comment_col) {
      $colomns['comment'] = array(
        'description' => "Whether comments are allowed on this {$this->label}" . $rev_data . ': 0 = no, 1 = closed (read only), 2 = open (read/write).',
        'type' => 'int',
        'not null' => TRUE,
        'default' => 0,
      );
    }

    if ($this->status_col) {
      $colomns['status'] = array(
        'description' => "Boolean indicating whether the {$this->label}" . $rev_data . ' is published (visible to non-administrators).',
        'type' => 'int',
        'not null' => TRUE,
        'default' => 1,
      );
    }

    if ($this->promote_col) {
      $colomns['promote'] = array(
        'description' => "'Boolean indicating whether the {$this->label}" . $rev_data . ' should be displayed on the front page.',
        'type' => 'int',
        'not null' => TRUE,
        'default' => 0,
      );
    }

    if ($this->sticky_col) {
      $colomns['sticky'] = array(
        'description' => "Boolean indicating whether the {$this->label}" . $rev_data . ' should be displayed at the top of lists in which it appears.',
        'type' => 'int',
        'not null' => TRUE,
        'default' => 0,
      );
    }

    // If I want to make survey entity exportable I need to add thoses fields.
    // Edit the entity keys for changing the default export status name, to avoid conflict with the status colomn.
    $colomns['export_status'] = array(
      'type' => 'int',
      'not null' => TRUE,
      // Set the default to ENTITY_CUSTOM without using the constant as it is
      // not safe to use it at this point.
      'default' => 0x01,
      'size' => 'tiny',
      'description' => 'The exportable status of the entity.',
    );
    $colomns['export_module'] = array(
      'description' => 'The name of the providing module if the entity has been defined in code.',
      'type' => 'varchar',
      'length' => 255,
      'not null' => FALSE,
    );

    // @TODO : add tnid and translate ?


    return $colomns;
  }

  /**
   * Define schema for custom fields.
   *
   * each class should implement it.
   *
   * @return
   *   An array, the custom schema fields or an empty array if there is
   *   no fields.
   *
   * @see LimesurveySyncDpInfos::schema()
   */
  protected function schema_custom_fields($revision = FALSE, $datas = array()) {
    return array();
  }

  /**
   * Provide hook_entity_property_info() for the entity.
   *
   * Thoses informations will be used into views, rules, token modules and the
   * entity view page.
   *
   * @return
   *   An array, the entity_property_info() informations.
   *
   *   @see hook_entity_property_info()
   */
  public function entity_property_info($datas = array()) {

    // Create the first entity name : register starting a survey from Drupal.
    $entity_name = $this->entityname;
    $special_fields = array_merge($this->entity_property_info_dp_fields(), $this->entity_property_info_custom_fields($datas));
    // Set the url.
    $arg = array('%type' => $this->label);
    $url = array(
      'url' => array(
        'label' => t('URL'),
        'description' => t('The URL of the %type', $arg),
        'getter callback' => 'entity_metadata_entity_get_properties',
        'type' => 'uri',
        'computed' => 1,
        'entity views field' => 1,
      ),
    );
    // Add the ls_entity_type_field.
    //$ls_entity_type_field = array(
//  'ls_entity_type' => array(
//    'label' => t('the entity type'),
//    'description' => t('The type of entity (%type)', $arg),
//    'type' => 'text',
//    'entity views field' => TRUE,
//  ),
    //);

    $property_info[$entity_name]['properties'] = array_merge($url, $special_fields);

    return $property_info;
  }

  /**
   * Provided entity_property_info() informations for DP fields.
   *
   * DP fields are 'uid', 'created', 'changed', 'vid', etc...
   *
   * @return
   *   An array, the DP entity_property_info() fields.
   *
   * @see LimesurveySyncDpInfos::entity_property_info()
   */
  protected function entity_property_info_dp_fields($revision = FALSE) {
    // No html for label.

    $info = array();
    $arg_label = array('@type' => $this->label);
    $arg_desc = array(
      '%type' => $this->label,
      '%types' => $this->plural_label,
    );

    // Set primary key.
    $primary_field = array(
      $this->primary_key => array(
        'label' => t('@type ID', $arg_label),
        'description' => t('Internal, numeric ID.'),
        'type' => 'integer',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => $this->primary_key,
      ),
    );
    // set date fields.
    if ($this->date_col) {
      $info['created'] = array(
        'label' => t('@type created', $arg_label),
        'description' => t('The Unix timestamp when the %type was created.', $arg_desc),
        'type' => 'date',
        'schema field' => 'created',
      );
      $info['changed'] = array(
        'label' => t('@type changed date', $arg_label),
        'description' => t('The Unix timestamp when the %type was most recently saved.', $arg_desc),
        'type' => 'date',
        'schema field' => 'created',
      );
    }

    // Set user field
    if ($this->user_col) {
      $info['uid'] = array(
        'label' => t('@type author', $arg_label),
        'description' => t('The author of the %type.', $arg_desc),
        'type' => 'user',
        'schema field' => 'uid',
      );
    }

    if ($this->language_col) {
      $info['language'] = array(
        'label' => t('@type language', $arg_label),
        'description' => t('The %type language.', $arg_desc),
        'type' => 'text',
        'schema field' => 'language',
      );
    }

    // set the vid field.
    if ($this->revision) {

      $info['vid'] = array(
        'label' => t('The @type version ID', $arg_label),
        'description' => t('The %type version ID.', $arg_desc),
        'type' => 'text',
        'schema field' => 'vid',
      );

      $info['log'] = array(
        'label' => t("revision log message"),
        'type' => 'text',
        'description' => t("In case a new revision is to be saved, the log entry explaining the changes for this version."),
        'schema field' => 'log',
        //'setter callback' => 'entity_property_verbatim_set',
        //'access callback' => '_limesurvey_sync_api_revision_access',
      );

    }

    if ($this->title_col) {
      $info['title'] = array(
        'label' => t('@type title', $arg_label),
        'description' => t('The %type title.', $arg_desc),
        'type' => 'text',
        'schema field' => 'title',
      );
    }

    $rev_data = ($revision) ? ' (at the time of this revision)' : '';

    if ($this->comment_col) {
      $info['comment'] = array(
        'label' => t('@type comment', $arg_label),
        'description' => 'Whether comments are allowed on this entity' . $rev_data . ': 0 = no, 1 = closed (read only), 2 = open (read/write).',
        'type' => 'integer',
        'schema field' => 'comment',
      );
    }

    if ($this->status_col) {
      $info['status'] = array(
        'label' => t('@type status', $arg_label),
        'description' => 'Boolean indicating whether the entity' . $rev_data . ' is published (visible to non-administrators).',
        'type' => 'integer',
        'schema field' => 'status',
      );
    }

    if ($this->promote_col) {
      $info['promote'] = array(
        'label' => t('@type promote', $arg_label),
        'description' => 'Boolean indicating whether the entity' . $rev_data . ' should be displayed on the front page.',
        'type' => 'integer',
        'schema field' => 'promote',
      );
    }

    if ($this->sticky_col) {
      $info['sticky'] = array(
        'label' => t('@type sticky', $arg_label),
        'description' => 'Boolean indicating whether the entity' . $rev_data . ' should be displayed at the top of lists in which it appears.',
        'type' => 'integer',
        'schema field' => 'sticky',
      );
    }

    if ($this->type_col_bundle) {
      $info['type'] = array(
        'label' => t('@type type', $arg_label),
        'description' => t('The machine name of the %type.', $arg_desc),
        'type' => 'token',
        'schema field' => 'type',
      );
    }

    if ($this->type_col_entity && !$revision) {
      $info['type'] = array(
        'label' => t('@type type', $arg_label),
        'description' => t('The survey ID into the Drupal software'),
        'type' => $this->has_bundle,
        'schema field' => 'type',
        'required' => TRUE,
        'bundle' => $this->has_bundle,
      );
    }

    if ($this->label_col) {
      $info['label'] = array(
        'label' => t('@type Title', $arg_label),
        'description' => t('The human readable name of the %type.', $arg_desc),
        'type' => 'text',
        'schema field' => 'label',
      );
    }
    if ($this->weight_col) {
      $info['weight'] = array(
        'label' => t('@type weight', $arg_label),
        'description' => t('A value for sorting %types.', $arg_desc),
        'type' => 'integer',
        'schema field' => 'weight',
      );
    }
    if ($this->description_col) {
      $info['description'] = array(
        'label' => t('@type description', $arg_label),
        'description' => t('A brief description of this %type.', $arg_desc),
        'type' => 'text',
        'schema field' => 'description',
      );
    }
    if ($this->help_col) {
      $info['help'] = array(
        'label' => t('@type help', $arg_label),
        'description' => t('Help information shown to the user when creating a content of this %type.', $arg_desc),
        'type' => 'text',
        'schema field' => 'help',
      );
    }

    return $info;
  }

  /**
   * Provided entity_property_info() informations for custom fields.
   *
   * Each class should implement it.
   *
   * @return
   *   An array, the DP entity_property_info() fields.
   *
   * @see LimesurveySyncDpInfos::entity_property_info()
   */
  protected function entity_property_info_custom_fields($datas = array()) {
    return array();
  }

  /**
   * Provide default values on creating entity.
   *
   * @return
   *   An array, the defaut values keyed by there field name.
   */
  public function entity_default_values() {

    $custom_fields = $this->entity_default_values_custom();

    $default_values = array();
    // set primary field
    if (!empty($this->primary_key)) {
      $default_values[$this->primary_key] = NULL;
    }
    // set date fields.
    if ($this->date_col) {
      // 'created' and 'changed' fields are handled into the save() method.
      $default_values['created'] = NULL;
      $default_values['changed'] = NULL;
    }

    // Set user field
    if ($this->user_col) {
      // 'user' field is handled into the save() method.
      global $user;
      $default_values['uid'] = $user->uid;
    }

    if ($this->language_col) {
      // 'language' field is handled into the save() method.
      global $language;
      $default_values['language'] = $language->language;
    }
    // set the vid field.
    if ($this->revision) {
      // 'vid' field is handled into the save() method.
      $default_values['vid'] = NULL;
    }
    if ($this->label_col) {
      $default_values['label'] = '';
    }
    if ($this->type_col_entity) {
      $default_values['type'] = '';
    }
    if ($this->type_col_bundle) {
      $default_values['type'] = '';
    }
    if ($this->weight_col) {
      $default_values['weight'] = 0;
    }
    if ($this->description_col) {
      $default_values['description'] = '';
    }
    if ($this->help_col) {
      $default_values['help'] = '';
    }
    if ($this->status_col) {
      $default_values['status'] = NODE_PUBLISHED;
    }

    return array_merge($default_values, $custom_fields);

  }

  /**
   * Provide default values on entity custom fields.
   *
   * Each class should implement it.
   *
   * @return
   *   An array, the defaut values keyed by there field name.
   */
  protected function entity_default_values_custom($datas = array()) {
    return array();
  }

  /**
   * Provide hook_permission for the entity.
   *
   * @return
   *   An array, the entity permissions.
   *
   * @see hook_permission()
   */
  public function permission($datas = array()) {
    // We set up permisssions to manage entity types, manage all entities and the
    // permissions for each individual entity
    // @TODO : need to translate this throw an handler.
    $arg = array(
      '@type' => $this->label,
      '@types' => $this->plural_label,
    );

    $permissions = array(
      "administer {$this->entityname}" => array(
        'title' => t('Administer @types', $arg),
        'description' => t('Create and delete @types, and set their permissions.', $arg),
      ),
    );

    $perm_overview = "access {$this->entityname} overview";
    $permissions += array(
      $perm_overview => array(
        'title' => t('Access the @types overview page', $arg),
        'description' => user_access($perm_overview)
        ? t('Get an overview of <a href="@url">all @types</a>.', array_merge($arg, array('@url' => url($this->overview_path))))
        : t('Get an overview of all @types.', $arg),
      ),
    );

    // Manage bundles.
    $has_bundle = FALSE;
    if ($this->has_bundle) {
      $has_bundle = TRUE;
      $bundle_entity_name = $this->has_bundle;
      $bundles = entity_load($bundle_entity_name);
    }
    else {
      $bundles = array('one');
    }

    // Handle bundles for permissions.
    foreach ($bundles as $bundle) {

      $perm_machine_suffix = ($has_bundle) ?  "{$bundle->type} {$this->entityname}" : "{$this->entityname}";
      $perm_human_prefix =  ($has_bundle) ? t('%type_name:', array('%type_name' => $bundle->label)) . ' ' :  '';
      // Adding answer or token entities from user interface is not possible.
      // I may active it if dp2ls synchronization is enable.
      if ($this->perm_add) {
        $permissions += array(
          "add {$perm_machine_suffix}" => array(
            'title' => $perm_human_prefix . t('Create new @type', $arg),
          ),
        );
      }

      // Update permission.
      if ($this->perm_update) {
        if ($this->user_col) {
          $permissions += array(
            "update own {$perm_machine_suffix}" => array(
              'title' => $perm_human_prefix . t('Edit own @type', $arg),
            ),
            "update any {$perm_machine_suffix}" => array(
              'title' => $perm_human_prefix . t('Edit any @type', $arg),
            ),
          );
        }
        else {
          $permissions += array(
            "update {$perm_machine_suffix}" => array(
              'title' => $perm_human_prefix . t('Edit @type', $arg),
            ),
          );
        }
      }

      // View permission.
      if ($this->perm_view) {
        if ($this->user_col) {
          $permissions += array(
            "view own {$perm_machine_suffix}" => array(
              'title' => $perm_human_prefix . t('View own @type', $arg),
            ),
            "view any {$perm_machine_suffix}" => array(
              'title' => $perm_human_prefix . t('View any @type', $arg),
            ),
          );
        }
        else {
          $permissions += array(
            "view {$perm_machine_suffix}" => array(
              'title' => $perm_human_prefix . t('View @type', $arg),
            ),
          );
        }
      }

      // Delete permission.
      if ($this->perm_delete) {
        if ($this->user_col) {
          $permissions += array(
            "delete own {$perm_machine_suffix}" => array(
              'title' => $perm_human_prefix . t('Delete own @type', $arg),
            ),
            "delete any {$perm_machine_suffix}" => array(
              'title' => $perm_human_prefix . t('Delete any @type', $arg),
            ),
          );
        }
        else {
          $permissions += array(
            "delete {$perm_machine_suffix}" => array(
              'title' => $perm_human_prefix . t('Delete @type', $arg),
            ),
          );
        }
      }

    }

    if ($this->status_col) {
      $permissions += array(
        "view own unpublished {$this->entityname}" => array(
          'title' => t('view own unpublished @type', $arg),
        ),
      );
    }

    if ($this->revision) {
      // Handle revision permissions. No bundle consideration.
      $perm_machine_suffix = "{$this->entityname}";
      $perm_human_prefix =  '';

      $permissions += array(
        "view {$perm_machine_suffix} revisions" => array(
          'title' => $perm_human_prefix . t('View any @type revisions', $arg),
        ),
        "revert {$perm_machine_suffix} revisions" => array(
          'title' => $perm_human_prefix . t('Revert any @type revisions', $arg),
        ),
        "delete {$perm_machine_suffix} revisions" => array(
          'title' => $perm_human_prefix . t('Delete any @type revisions', $arg),
        ),
      );
    }

    // Custom permissions
    $custom_permissions = $this->permission_custom($datas);
    $permissions = array_merge($permissions, $custom_permissions);

    return $permissions;
  }

  /**
   * Define custom permissions.
   *
   * each class should implement it.
   *
   * @return
   *   An array, the entity permissions.
   *
   * @see LimesurveySyncDpInfos::permission()
   */
  protected function permission_custom($datas = array()) {
    return array();
  }

}

/**
 *  A controller implementing exportables stored in the database.
 *
 * Extend the common class 'EntityAPIControllerExportable' for entities.
 */
class LimesurveySyncEntity extends Entity {
  /**
   * Returns the uri of the entity just as entity_uri().
   *
   * Modules may alter the uri by specifying another 'uri callback' using
   * hook_entity_info_alter(). Set uri from LimesurveySyncInfos::crud path value.
   *
   * @see entity_uri()
   */
  protected function defaultUri() {
    $id = $this->identifier();
    if (!is_numeric($id)) {
      $id = str_replace('_', '-', $id);
    }
    return array(
      'path' => $this->entityInfo['crud path'] . "/{$id}",
    );
  }


}

/**
 * Class managing entities.
 *
 * Extend the common class 'Entity' for entities.
 */
class LimesurveySyncEntityController extends EntityAPIControllerExportable {
  /**
   * Overridden.
   *
   * Set default values on creating entity.
   */
  public function create(array $values = array()) {
    // Add values that are specific to our LimesurveySyncSurvey
    $entity_infos = limesurvey_sync_api_info_class_object($this->ls_entity_type);
    $fields = $entity_infos->entity_default_values();
    $values += $fields;
    $entity = parent::create($values);
    return $entity;
  }

  /**
   * Overridden.
   *
   * Add The entity_type name in the entity. issue https://www.drupal.org/node/1042822
   */
  public function load($ids = array(), $conditions = array()) {

    // https://www.drupal.org/node/1042822#comment-6688828
    $entities = parent::load($ids, $conditions);
    foreach ($entities as $entity) {
      $entity->entity_type = $this->entityType;
    }
    return $entities;
  }


  public function save($entity, DatabaseTransaction $transaction = NULL) {
    $insert = !empty($entity->is_new);
    if (in_array('created', $this->entityInfo['schema_fields_sql']['base table'])) {
      // Care about setting created and changed values. But do not automatically
      // set a created values for already existing entity.
      if (empty($entity->created) && !empty($entity->is_new)) {
        $entity->created = REQUEST_TIME;
      }
    }
    if (in_array('changed', $this->entityInfo['schema_fields_sql']['base table'])) {
      $entity->changed = REQUEST_TIME;
    }
    if (in_array('status', $this->entityInfo['schema_fields_sql']['base table']) && !empty($entity->is_new)) {
      $entity->status = ENTITY_CUSTOM;
      $entity->module = $this->entityInfo['module'];
    }

    if (!empty($this->entityInfo['entity keys']['revision'])) {
      if (in_array('timestamp', $this->entityInfo['schema_fields_sql']['revision table'])) {
        $entity->timestamp = REQUEST_TIME;
      }

      if (in_array('log', $this->entityInfo['schema_fields_sql']['revision table'])) {

        if ((!empty($entity->is_new)) || !empty($entity->revision)) {
          // When inserting either a new entity or a new node revision, $entity->log
          // must be set because {revision}.log is a text column and therefore
          // cannot have a default value. However, it might not be set at this
          // point (for example, if the user submitting a entity form does not have
          // permission to create revisions), so we ensure that it is at least an
          // empty string in that case.
          // @todo: Make the {revision}.log column nullable so that we can
          // remove this check.
          if (!isset($entity->log)) {
            $entity->log = '';
          }
        }
        elseif (!isset($entity->log) || $entity->log === '') {
          // If we are updating an existing entity without adding a new revision, we
          // need to make sure $entity->log is unset whenever it is empty. As long as
          // $entity->log is unset, drupal_write_record() will not attempt to update
          // the existing database column when re-saving the revision; therefore,
          // this code allows us to avoid clobbering an existing log entry with an
          // empty one.
          unset($entity->log);
        }
      }
    }

    if (in_array('uid', $this->entityInfo['schema_fields_sql']['base table'])) {
      global $user;
      if (!isset($entity->uid) && !empty($entity->is_new)) {
        $entity->uid = 0;
      }
      $entity->revision_uid = $user->uid;
    }

    if (in_array('language', $this->entityInfo['schema_fields_sql']['base table'])) {
      if (empty($entity->language) && !empty($entity->is_new)) {
        global $language;
        $entity->language = $language->language;
      }
    }
    // Use entity_revision_set_default($entity_type, $entity) for make-current.

    $return = parent::save($entity, $transaction);

    // Watchdog.
    // Handle bundle if any.
    $human_bundle_label = '';
    if (!empty($this->entityInfo['entity keys']['bundle']) && !empty($this->entityInfo['bundles'])) {
      // The entity has bundles.
      if ($entity) {
        $type = $entity->{$this->entityInfo['entity keys']['bundle']};
        $bundle_entity_name = limesurvey_sync_api_entity_find_bundle_entity_type($this->entityType);
        if ($bundle_entity_name = limesurvey_sync_api_entity_find_bundle_entity_type($this->entityType)) {
          $bundle = $entity->{$this->entityInfo['entity keys']['bundle']};
          $human_bundle_label = entity_label($bundle_entity_name, $bundle);
        }
      }
    }

    $name = (!empty($human_bundle_label)) ? $human_bundle_label . $this->entityInfo['label'] : $this->entityInfo['label'];
    $watchdog_args = array(
      '@type' => drupal_strtoupper(drupal_substr($name, 0, 1)) . drupal_substr($name, 1),
      '%title' => entity_label($this->entityType, $entity),
    );
    $link = l(t('view'), current(entity_uri($this->entityType, $entity)));
    if (in_array($return, array(SAVED_NEW, SAVED_UPDATED))) {
      $severity = WATCHDOG_NOTICE;
      $message = ($insert) ? '@type: added %title.' : '@type: updated %title.';
    }
    else {
      $severity = WATCHDOG_ERROR;
      $message = '@type: saving process has failed for %title';
    }
    watchdog($this->entityInfo['module'], $message, $watchdog_args, $severity, $link);
  }
}

/**
 * Default controller for providing UI.
 *
 * Extend the common class 'EntityDefaultUIController' for entities.
 */
class LimesurveySyncEntityUIController extends EntityDefaultUIController {
  // CRUD info for building menu. They are build from permissions and overwrited
  // by the child class.
  protected $allow_add = NULL;
  protected $allow_update = NULL;
  protected $allow_view = NULL;
  protected $allow_delete = NULL;
  protected $has_view_page = NULL;
  protected $hide_export_status = FALSE;

  /**
   * retrieve crud values from the permissions.
   */
  public function __construct($entity_type, $entity_info) {
    parent::__construct($entity_type, $entity_info);
    $permissions = module_invoke($this->entityInfo['module'], 'permission');
    $crud = array('add', 'update', 'view', 'delete');
    $length = -1 * (drupal_strlen($this->entityType) + 1);
    foreach ($permissions as $machine_permission => $info) {
      foreach ($crud as $op) {
        $variable = "allow_{$op}";
        if ($this->$variable === NULL && drupal_substr($machine_permission, $length) == " {$this->entityType}" && drupal_substr($machine_permission, 0, (drupal_strlen($op) + 1)) == "{$op} ") {
          $this->$variable = TRUE;
        }
      }
    }
    foreach ($crud as $op) {
      $variable = "allow_{$op}";
      if ($this->$variable === NULL) {
        $this->$variable = FALSE;
      }
    }

    // For Bundle : As an exemple, node types have no view page, there "view"
    // page are redirected to edit page.
    if ($this->has_view_page === NULL) {
      $this->has_view_page = (empty($this->entityInfo['bundle of']));
    }
  }

  /**
   * Provides definitions for implementing hook_menu().
   *
   * Manage entity view page, adding, synchronize (one and all) pages, revisions,
   * import, clone, devel pages.
   */
  public function hook_menu() {

    // In order to works with the menu router, the modules must be called in
    // this order : bundle, theh the entity of this bundle, depending on the
    // order in the hook_entity_info() functions are called.
    $items = parent::hook_menu();

    // Edit result from EntityDefaultUIController::hook_menu().
    $wildcard = isset($this->entityInfo['admin ui']['menu wildcard']) ? $this->entityInfo['admin ui']['menu wildcard'] : '%entity_object';

    // All menu below are generated by EntityDefaultUIController::hook_menu()
    // and may need adaptations.
    $old_path_overview = $this->path; // overview page
    $old_path_overview_default = "{$this->path}/list";
    $old_path_add = "{$this->path}/add";
    $old_path_import = "{$this->path}/import";
    $old_path_edit = "{$this->path}/manage/{$wildcard}";
    $old_path_edit_default = "{$this->path}/manage/{$wildcard}/edit";
    $old_path_clone = "{$this->path}/manage/{$wildcard}/clone";
    $old_path_revert_delete = "{$this->path}/manage/{$wildcard}/%";
    //$old_path_export = "{$this->path}/manage/{$wildcard}/export";

    //$new_path_view = "{$this->entityInfo['crud path']}/{$wildcard}";

    $new_items = array();

    // Fix the add page : from the admin menu to general menu; use
    // hook_menu_local_tasks_alter() to add it to admin menu and
    // hook_admin_paths() pour admin theme.
    // The add page for response, is handled into the response page.


    if (isset($items[$old_path_import])) {
      $new_path_import = "{$this->entityInfo['crud path']}/import";
      $new_items[$new_path_import] = $items[$old_path_import];
      // Hide "/import" into the menu.
      $items["{$this->path}/import"]['type'] = MENU_CALLBACK;
    }

    if (isset($items[$old_path_overview])) {
      $new_path_overview = $this->entityInfo['overview path'];
      $new_items[$new_path_overview] = $items[$old_path_overview];
      $new_items["{$new_path_overview}/list"] = $items[$old_path_overview_default];
    }

    // Handle Add entity :
    if ($this->allow_add) {

      $has_bundles = !empty($this->entityInfo['entity keys']['bundle']);
      if ($has_bundles) {
        // Create direct link if there is only one bundle.
        $new_items["{$this->entityInfo['crud path']}/add"] = array(
          'title' => 'Add response',
          'page callback' => 'node_add_page',
          'access callback' => '_limesurvey_sync_api_entity_bundle_add_access',
          'access arguments' => array($this->entityType, NULL),
          'file' => 'node.pages.inc',
          'file path' => drupal_get_path('module', 'node'),
        );
        // Create an add menu item for each bundle.
        foreach ($this->entityInfo['bundles'] as $type => $datas) {
          $type_url_str = str_replace('_', '-', $type);
          $new_path_add = "{$this->entityInfo['crud path']}/add/{$type_url_str}";
          $new_items[$new_path_add] = $items[$old_path_add];
          $new_items[$new_path_add]['title arguments'][2] = $type;
          // Set My special callback and arguements
          $new_items[$new_path_add]['page callback'] = 'limesurvey_sync_api_entity_entity_ui_get_form';
          $new_items[$new_path_add]['page arguments'] = array($this->entityType, NULL, 'add', array(), $type);
          $new_items[$new_path_add]['access callback'] = '_limesurvey_sync_api_entity_bundle_add_access';
          $new_items[$new_path_add]['access arguments'] = array($this->entityType, $type);

          // Add this for the menu navigation from the overview page.
          // No : done into limesurvey_sync_menu_local_tasks_alter().
          //$new_path_add2 = "{$this->entityInfo['overview path']}/add/{$type_url_str}";
          //$new_items[$new_path_add2] = $new_items[$new_path_add];
          //$new_items[$new_path_add2]['page callback'] = 'drupal_goto';
          //$new_items[$new_path_add2]['page arguments'] = array($new_path_add);
        }

      }
      else {
        if (isset($items[$old_path_add])) {
          $new_path_add = "{$this->entityInfo['crud path']}/add";
          $new_items[$new_path_add] = $items[$old_path_add];
          // Usemy callback for autsetting form
          $new_items[$new_path_add]['page callback'] = 'limesurvey_sync_api_entity_entity_ui_get_form';
        }
      }
    }

    $new_path_view = "{$this->entityInfo['crud path']}/{$wildcard}";
    $id_count = count(explode('/', $new_path_view));
    $placeholder_view = $id_count - 1;

    // There is no view page. We create it
    if ($this->has_view_page) {
      //{$this->path}/manage/{$wildcard} and {$this->path}/manage/{$wildcard}/edit
      // need to be replaced
      $new_path_view = "{$this->entityInfo['crud path']}/{$wildcard}";
      $id_count = count(explode('/', $new_path_view));
      $placeholder_view = $id_count - 1;
      $new_items[$new_path_view] = array(
        'title callback' => 'entity_label',
        'title arguments' => array(
          $this->entityType,
          $placeholder_view,
        ),
        'access callback' => 'entity_access',
        'access arguments' => array(
          'view',
          $this->entityType,
          $placeholder_view,
        ),
        'page callback' => 'limesurvey_sync_api_entity_view_page',
        'page arguments' => array(
          $this->entityType,
          $placeholder_view,
        ),
        'load arguments' => array($this->entityType),
        'description' => "View {$this->entityInfo['label']} page",
        'type' => MENU_CALLBACK,
      );

      $new_items["{$new_path_view}/view"] = array(
        'title' => 'view',
        'weight' => -10,
        'type' => MENU_DEFAULT_LOCAL_TASK,
      );

      // Change Edit page path.
      if (isset($items[$old_path_edit])) {
        $new_path_edit = "{$new_path_view}/edit";
        $new_items[$new_path_edit] = $items[$old_path_edit];
        // Adapt it. Change placeholder.
        $new_items[$new_path_edit]['page arguments'][1] = $placeholder_view;
        $new_items[$new_path_edit]['access arguments'][2] = $placeholder_view;
        // set the edit page as tab.
        $new_items[$new_path_edit]['type'] = MENU_LOCAL_TASK;
        // Title of the edi tab is simply "Edit".
        unset($new_items[$new_path_edit]['title arguments'], $new_items[$new_path_edit]['title callback']);
      }
    }
    else {
      // The entity has no view page (as node type).
      $new_path_edit = $new_path_view;
      $new_items[$new_path_edit] = $items[$old_path_edit];
      $new_items["{$new_path_edit}/edit"] = $items[$old_path_edit_default];
      if ($new_path_view != $old_path_edit) {
        $new_items[$new_path_edit]['title arguments'][1] = $placeholder_view;
        $new_items[$new_path_edit]['page arguments'][1] = $placeholder_view;
        $new_items[$new_path_edit]['access arguments'][2] = $placeholder_view;
      }
    }

    // Change the page callback for editing entities
    if (isset($new_items[$new_path_edit])) {
      // This will allow to display automatically stuff for edition (buttons, etc...)
      $new_items[$new_path_edit]['page callback'] = 'limesurvey_sync_api_entity_entity_ui_get_form';
    }

    if (isset($items[$old_path_clone])) {
      $new_path_clone = "{$new_path_view}/clone";
      $new_items[$new_path_clone] = $items[$old_path_clone];
      $new_items[$new_path_clone]['page arguments'][1] = $placeholder_view;
      // Use My callback
      $new_items[$new_path_clone]['page callback'] = 'limesurvey_sync_api_entity_entity_ui_get_form';
      // Remove from the menu.
      $new_items[$new_path_clone]['type'] = MENU_CALLBACK;
    }

    if (isset($items[$old_path_revert_delete])) {
      $new_path_revert_delete = "{$new_path_view}/%";
      $new_items[$new_path_revert_delete] = $items[$old_path_revert_delete];
      $new_items[$new_path_revert_delete]['page arguments'][2] = $placeholder_view;
      $new_items[$new_path_revert_delete]['page arguments'][3] = ($placeholder_view + 1);
      $new_items[$new_path_revert_delete]['access arguments'][2] = $placeholder_view;
      unset($new_items[$new_path_revert_delete]['load arguments']);
      // Remove from the menu.
      $new_items[$new_path_revert_delete]['type'] = MENU_CALLBACK;
    }


    // Handle access :

    if (!$this->allow_view && isset($new_items[$new_path_view])) {
      unset($new_items[$new_path_view]);
    }
    if (!$this->allow_update && isset($new_items[$new_path_edit])) {
      unset($new_items[$new_path_edit]);
    }
    if (!$this->allow_delete && isset($new_items[$new_path_revert_delete])) {
      unset($new_items[$new_path_revert_delete]);
    }

    // Handle revision stuff.
    if (!empty($this->entityInfo['entity keys']['revision'])) {
      $new_items[$new_path_view . '/revisions'] = array(
        'title' => 'Revisions',
        'page callback' => 'limesurvey_sync_api_entity_revision_overview',
        'page arguments' => array(
          $this->entityType,
          $placeholder_view,
        ),
        'load arguments' => array($this->entityType),
        'access callback' => '_limesurvey_sync_api_entity_revision_access',
        'access arguments' => array(
          $this->entityType,
          $placeholder_view,
        ),
        'weight' => 2,
        'type' => MENU_LOCAL_TASK,
        'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api/',
        'file' => 'limesurvey_sync_api_entity.pages.inc',
      );

      // Too bad the %entity_revision wildcard doesn't work. It may be because
      // of the previous %entity_object wildcard. I manage it in the
      // _limesurvey_sync_api_revision_access() function.
      $new_items[$new_path_view . '/revisions/%/view'] = array(
        'title' => 'Revisions',
        'load arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
        ),
        'page callback' => 'limesurvey_sync_api_show',
        'page arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
          TRUE,
        ),
        'access callback' => '_limesurvey_sync_api_entity_revision_access',
        'access arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
        ),
      );
      $new_items[$new_path_view . '/revisions/%/revert'] = array(
        'title' => 'Revert to earlier revision',
        'load arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
        ),
        'page callback' => 'drupal_get_form',
        'page arguments' => array(
          'limesurvey_sync_api_entity_revision_revert_confirm',
          $this->entityType,
          $placeholder_view + 2,
        ),
        'access callback' => '_limesurvey_sync_api_entity_revision_access',
        'access arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
          'update',
        ),
        'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api/',
        'file' => 'limesurvey_sync_api_entity.pages.inc',
      );
      $new_items[$new_path_view . '/revisions/%/delete'] = array(
        'title' => 'Delete earlier revision',
        'load arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
        ),
        'page callback' => 'drupal_get_form',
        'page arguments' => array(
          'limesurvey_sync_api_entity_revision_delete_confirm',
          $this->entityType,
          $placeholder_view + 2,
        ),
        'access callback' => '_limesurvey_sync_api_entity_revision_access',
        'access arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
          'delete',
        ),
        'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api/',
        'file' => 'limesurvey_sync_api_entity.pages.inc',
      );
    }

    // Set devel module with wildcard.
    if (module_exists('devel')) {
      $devel_path = drupal_get_path('module', 'devel');
      $new_items[$new_path_view . '/devel'] = array(
        'title' => 'Devel',
        'page callback' => 'devel_load_object',
        'page arguments' => array(
          $this->entityType,
          $placeholder_view,
        ),
        'access arguments' => array('access devel information'),
        'load arguments' => array($this->entityType),
        'type' => MENU_LOCAL_TASK,
        'file' => 'devel.pages.inc',
        'weight' => 100,
        'file path' => $devel_path,
      );
      $new_items[$new_path_view . '/devel/load'] = array(
        'title' => 'Load',
        'type' => MENU_DEFAULT_LOCAL_TASK,
      );
      // @TODO : render page fails because it calls limesurvey_sync_token_view()
      // function that not exists (fata error).
      $new_items[$new_path_view . '/devel/render'] = array(
        'title' => 'Render',
        'page callback' => 'devel_render_object',
        'page arguments' => array(
          // For answer and token, provide 'limesurvey_sync_token' instead of
          // 'limesurvey_sync_token_xxxxxx' for calling limesurvey_sync_token_view().
          rtrim(limesurvey_sync_api_entityname($this->ls_entity_type, 'prefix'), '_'),
          $placeholder_view,
          $this->entityType,
        ),
        'access arguments' => array('access devel information'),
        'load arguments' => array($this->entityType),
        'file' => 'devel.pages.inc',
        'type' => MENU_LOCAL_TASK,
        'file path' => $devel_path,
        'weight' => 100,
      );
    }
    //}

    return $new_items;
  }

  /**
   * Generates the table headers for the overview table.
   */
  protected function overviewTableHeaders($conditions, $rows, $additional_header = array()) {

    $header = $additional_header;

    // Add the type colonne.
    $has_bundles = !empty($this->entityInfo['entity keys']['bundle']);
    if ($has_bundles) {
      $header[] = t('type');
    }
    $has_author = in_array('uid', $this->entityInfo['schema_fields_sql']['base table']);
    if ($has_author) {
      $header[] = t('User');
    }
    array_unshift($header, t('Label'));
    if (!empty($this->entityInfo['exportable']) && !$this->hide_export_status) {
      $header[] = t('Status');
    }
    // Add operations with the right colspan.
    $header[] = array(
      'data' => t('Operations'),
      'colspan' => $this->operationCount(),
    );
    return $header;
  }

  /**
   * Generates the row for the passed entity and may be overridden in order to
   * customize the rows.
   *
   * Add the synchronization status and LimeSurvey status colomns and the
   * synchronize operation. Delete export status colomn and edit operation
   * depending on settings ($hide_export_status and $disallow_edit).
   *
   * @param $additional_cols
   *   Additional columns to be added after the entity label column.
   */
  protected function overviewTableRow($conditions, $id, $entity, $additional_cols = array(), $additional_ops = array()) {

    // Disable stuff.
    //return parent::overviewTableRow($conditions, $id, $entity, $additional_cols);
    // Rewrite te whole method, since all links changes.
    if (!is_numeric($id)) {
      $id = str_replace('_', '-', $id);
    }

    $row = array();

    $entity_uri = entity_uri($this->entityType, $entity);

    $row[] = array(
      'data' => array(
        '#theme' => 'entity_ui_overview_item',
        '#label' => entity_label($this->entityType, $entity),
        '#name' => !empty($this->entityInfo['exportable']) ? entity_id($this->entityType, $entity) : FALSE,
        '#url' => $entity_uri ? $entity_uri : FALSE,
        '#entity_type' => $this->entityType,
      ),
    );

    // Add in any passed additional cols.
    foreach ($additional_cols as $col) {
      $row[] = $col;
    }

    $has_bundles = !empty($this->entityInfo['entity keys']['bundle']);
    if ($has_bundles) {
      $type = $entity->{$this->entityInfo['entity keys']['bundle']};
      foreach ($this->entityInfo['bundles'] as $bundle_type => $bundle_datas) {
        if ($bundle_type == $type) {
          $row[] = l($bundle_datas['label'], $bundle_datas['admin']['real path']);
        }
      }
    }
    $has_author = in_array('uid', $this->entityInfo['schema_fields_sql']['base table']);
    if ($has_author) {
      $uid = $entity->uid;
      $account = user_load($uid);
      $name =  format_username($account);
      $user_uri = (!empty($account->uid) && user_access('View user profiles')) ? "user/{$account->uid}" : FALSE;
      $row[] = $user_uri ? l($name, $user_uri) : $name;
    }


    // Add a row for the exportable status.
    if (!empty($this->entityInfo['exportable']) && !$this->hide_export_status) {
      $row[] = array('data' => array(
          '#theme' => 'entity_status',
          '#status' => $entity->{$this->statusKey},
        ));
    }

    foreach ($additional_ops as $op) {
      $row[] = $op;
    }

    // In case this is a bundle, we add links to the field ui tabs.
    $field_ui = !empty($this->entityInfo['bundle of']) && entity_type_is_fieldable($this->entityInfo['bundle of']) && module_exists('field_ui');
    // For exportable entities we add an export link.
    $exportable = !empty($this->entityInfo['exportable']);
    // If i18n integration is enabled, add a link to the translate tab.
    $i18n = !empty($this->entityInfo['i18n controller class']);

    // Set a view path instead of admin path:
    $path_view = $this->entityInfo['crud path'];

    // Add operations depending on the status.
    if (entity_has_status($this->entityType, $entity, ENTITY_FIXED)) {
      $row[] = array(
        'data' => l(t('clone'), $path_view . '/' . $id . '/clone'),
        'colspan' => $this->operationCount(),
      );
    }
    else {
      // Delete edit link.
      if (!$this->disallow_edit) {
        $row[] = l(t('edit'), $path_view . '/' . $id . '/edit');
      }
      if ($field_ui) {
        $row[] = l(t('manage fields'), $this->path . '/manage/' . $id . '/fields');
        $row[] = l(t('manage display'), $this->path . '/manage/' . $id . '/display');
      }
      if ($i18n) {
        $row[] = l(t('translate'), $path_view . '/' . $id . '/translate');
      }
      // Cloning a entry may be a problem for linking drupal and limesurvey...
      if ($exportable) {
        $row[] = l(t('clone'), $path_view . '/' . $id . '/clone');
      }

      if (empty($this->entityInfo['exportable']) || !entity_has_status($this->entityType, $entity, ENTITY_IN_CODE)) {
        $row[] = l(t('delete'), $path_view . '/' . $id . '/delete', array('query' => drupal_get_destination()));
      }
      elseif (entity_has_status($this->entityType, $entity, ENTITY_OVERRIDDEN)) {
        $row[] = l(t('revert'), $path_view . '/' . $id . '/revert', array('query' => drupal_get_destination()));
      }
      else {
        $row[] = '';
      }
    }
    if ($exportable) {
      $row[] = l(t('export'), $path_view . '/' . $id . '/export');
    }

    return $row;
  }

  /**
   * Returns the operation count for calculating colspans.
   */
  protected function operationCount() {

    $count = parent::operationCount();
    $count -= $this->disallow_edit ? 1 : 0;
    return $count;
  }
}

/**
 * return entity_view result, if the entity is provided as an object either as
 * primary numeric key.
 */
function limesurvey_sync_api_entity_view_page($entity_type, $entity, $view_mode = 'full', $langcode = NULL, $page = NULL) {

  // This function exists because the entity object must be keyed.
  $entities = array($entity);
  if (!is_object($entity) && !empty($entity)) {
    $entities = entity_load($entity_type, $entities);
  }
  return entity_view($entity_type, $entities, $view_mode, $langcode, $page);
}


/**
 * Determines whether the given user has access to an entity.
 *
 * @param $op
 *   The operation being performed. One of 'view', 'update', 'create' or
 *   'delete'.
 * @param $entity_type
 *   The entity type of the entity to check for.
 * @param $entity
 *   Optionally an entity to check access for. If no entity is given, it will be
 *   determined whether access is allowed for all entities of the given type.
 * @param $account
 *   The user to check for. Leave it to NULL to check for the global user.
 *
 * @return boolean
 *   Whether access is allowed or not. If the entity type does not specify any
 *   access information, NULL is returned.
 *
 * @see entity_type_supports()
 */
function limesurvey_sync_api_entity_entity_access($op, $entity = NULL, $account = NULL, $entity_type = NULL) {
  // Map the op value. EntityDefaultUIController:hook_menu() use 'add' and
  // 'update'. entity_access() will work even if 'create' or 'edit' are provided.
  switch ($op) {
    case 'edit':
      $op = 'update';
      break;
    case 'create':
      $op = 'add';
      break;
  }

  if ((empty($entity) && empty($entity_type)) || !in_array($op, array('view', 'update', 'delete', 'add'), TRUE)) {
    // If there was no entity to check against, or the $op was not one of the
    // supported ones, we return access denied.
    return FALSE;
  }

  // If no user object is supplied, the access check is for the current user.
  if (empty($account)) {
    $account = $GLOBALS['user'];
  }

  // Check for high level permission.
  $admin_perm = "administer {$entity_type}";
  if (user_access($admin_perm, $account)) {
    return TRUE;
  }

  // Handle bundle if any.
  $type_str = '';
  $entity_info = entity_get_info($entity_type);
  if (!empty($entity_info['entity keys']['bundle']) && !empty($entity_info['bundles'])) {
    // The entity has bundles.
    if ($entity) {
      $type = $entity->{$entity_info['entity keys']['bundle']};
      $type_str = $type . ' ';
    }
  }

  // Prepare the user_access() string.
  $map = array(
    'administer' => $admin_perm,
    'add' => "add {$type_str}{$entity_type}",
    'view' => "view {$type_str}{$entity_type}",
    'view own' => "view own {$type_str}{$entity_type}",
    'view any' => "view any {$type_str}{$entity_type}",
    'view unpublished' => "view unpublished {$entity_type}",
    'view own unpublished' => "view own unpublished {$entity_type}",
    'update' => "update {$type_str}{$entity_type}",
    'update own' => "update own {$type_str}{$entity_type}",
    'update any' => "update any {$type_str}{$entity_type}",
    'delete' => "delete {$type_str}{$entity_type}",
    'delete own' => "delete own {$type_str}{$entity_type}",
    'delete any' => "delete any {$type_str}{$entity_type}",
  );

  // Handle author.
  $own_exists = (in_array('uid', $entity_info['schema_fields_sql']['base table']));
  $author =  ($own_exists && $account->uid == $entity->uid && $account->uid != 0);

  // Handle operations.
  // Create entity.
  if ($op == 'add') {
    if (user_access($map['add'], $account)) {
      return TRUE;
    }
    return FALSE;
  }

  // View entity.
  if ($op == 'view') {
    // check for unpublished entity.
    // Handle published.
    $status_exists = in_array('status', $entity_info['schema_fields_sql']['base table']);
    $published = (!$status_exists || $entity->status == NODE_PUBLISHED);
    if ($status_exists && !$published) {
      if ((!$own_exists && user_access($map['view unpublished'], $account)) || ($own_exists && $author && user_access($map['view own unpublished'], $account))) {
        return TRUE;
      }
      else {
        return FALSE;
      }
    }
  }

  // View, delete, update entity.
  if ((!$own_exists && user_access($map[$op], $account)) || ($own_exists && user_access($map["$op any"], $account))) {
    return TRUE;
  }
  if ($own_exists && $author && user_access($map["$op own"], $account)) {
    return TRUE;
  }

  // At last.
  return FALSE;
}

/**
 * Access callback: Checks entity revision access.
 *
 * Inspired from _node_revision_access().
 *
 * @param $entity_type
 *   The entity type of the entity to check for.
 * @param $entity
 *   Optionally an entity to check access for. If no entity is given, it will be
 *   determined whether access is allowed for all entities of the given type.
 * @param $op
 *   (optional) The specific operation being checked. Defaults to 'view.'
 * @param object $account
 *   (optional) A user object representing the user for whom the operation is
 *   to be performed. Determines access for a user other than the current user.
 *
 * @return
 *   TRUE if the operation may be performed, FALSE otherwise.
 */
function _limesurvey_sync_api_entity_revision_access($entity_type, $entity = NULL, $op = 'view', $account = NULL) {

  if (is_numeric($entity)) {
    $entity = entity_revision_load($entity_type, $entity);
  }

  $entity_info = entity_get_info($entity_type);
  if (!$entity_info || empty($entity_info['entity keys']['revision'])) {
    return FALSE;
  }

  $vid_field = $entity_info['entity keys']['revision'];

  $access = &drupal_static(__FUNCTION__, array());

  $map = array(
    'administer' => "administer {$entity_type}",
    'view' => "view {$entity_type} revisions",
    'update' => "revert {$entity_type} revisions",
    'delete' => "delete {$entity_type} revisions",
  );

  if (!$entity || !isset($map[$op])) {
    // If there was no entity to check against, or the $op was not one of the
    // supported ones, we return access denied.
    return FALSE;
  }

  if (!isset($account)) {
    $account = $GLOBALS['user'];
  }

  // Statically cache access by revision ID, user account ID, and operation.
  $cid = $entity->$vid_field . ':' . $account->uid . ':' . $op;

  if (!isset($access[$cid])) {
    // Perform basic permission checks first.
    if (!user_access($map[$op], $account) && !user_access($map['administer'], $account)) {
      return $access[$cid] = FALSE;
    }

    $id_field = $entity_info['entity keys']['id'];
    $entity_current_revisions = entity_load($entity_type, array($entity->$id_field));
    $entity_current_revision = $entity_current_revisions[$entity->$id_field];
    $is_current_revision = entity_revision_is_default($entity_type, $entity);

    // There should be at least two revisions. If the vid of the given entity and
    // the vid of the current revision differ, then we already have two
    // different revisions so there is no need for a separate database check.
    // Also, if you try to revert to or delete the current revision, that's not
    // good.

    if ($is_current_revision && (db_query('SELECT COUNT(' . $vid_field . ') FROM {' . $entity_info['revision table'] . '} WHERE ' . $id_field . ' = :' . $id_field, array(
      ":{$id_field}" => $entity->$id_field,
    ))->fetchField() == 1 || $op == 'update' || $op == 'delete')) {
      $access[$cid] = FALSE;
    }
    elseif (user_access($map['administer'], $account)) {
      $access[$cid] = TRUE;
    }
    else {
      // First check the access to the current revision and finally, if the entity
      // passed in is not the current revision then access to that, too.
      $access[$cid] = entity_access($op, $entity_type, $entity_current_revision, $account) && ($is_current_revision || entity_access($op, $entity_type, $entity, $account));
    }

    return $access[$cid];
  }
}

/**
 * Generates an array which displays a entity detail page.
 *
 * Inspired from node_show().
 *
 * @param $entity_type
 *   The entity type of the entity to check for.
 * @param $entity
 *   Optionally an entity to check access for. If no entity is given, it will be
 *   determined whether access is allowed for all entities of the given type.
 * @param $message
 *   A flag which sets a page title relevant to the revision being viewed.
 *
 * @return
 *   A $page element suitable for use by drupal_render().
 */
function limesurvey_sync_api_entity_show($entity_type, $entity, $message = FALSE) {

  if (is_numeric($entity)) {
    $entity = entity_revision_load($entity_type, $entity);
  }

  $entity_info = entity_get_info($entity_type);
  $id_field = $entity_info['entity keys']['id'];
  $ts_field = (in_array('timestamp', $this->entityInfo['schema_fields_sql']['revision table'])) ? 'timestamp' : FALSE;
  if ($message) {
    drupal_set_title(t('Revision of %title from %date', array(
      '%title' => entity_label($entity_type, $entity),
      '%date' => (($ts_field) ? format_date($entity->{$ts_field}) : t('unknown date')),
    )), PASS_THROUGH);
  }

  // For markup consistency with other pages, use node_view_multiple() rather than node_view().
  $entities = entity_view($entity_type, array(
    $entity->$id_field => $entity,
  ), 'full');

  // Update the history table, stating that this user viewed this entity.
  // entity_tag_new($entity);

  return $entities;
}

/**
 * Inspired by the _node_add_access() core function
 */
function _limesurvey_sync_api_entity_bundle_add_access($entity_type, $bundle_type) {
  $entity_info = entity_get_info($entity_type);
  if (!$entity_info || empty($entity_info['entity keys']['bundle'])) {
    return FALSE;
  }

  if (!empty($bundle_type)) {
    $types = array($bundle_type);
  }
  else {
    $types = array_keys($entity_info['bundles']);
  }
  foreach ($types as $type) {
    $entity = entity_create($entity_type, array($entity_info['entity keys']['bundle'] => $type));
    if (entity_access('add', $entity_type, $entity)) {
      return TRUE;
    }
  }

  foreach (entity_get_info() as $datas) {
    if (!empty($datas['bundle of']) && $datas['bundle of'] == $entity_type) {
      if (user_access("administer {$datas['bundle of']}")) {
        // There are no content types defined that the user has permission to create,
        // but the user does have the permission to administer the content types, so
        // grant them access to the page anyway.
        return TRUE;
      }
    }
  }

  return FALSE;
}

/**
 * Inspired by the node_form() core function
 */
function limesurvey_sync_api_entity_edit_form(&$form, $form_state, $entity, $entity_type) {
  $entity_info = entity_get_info($entity_type);
  if (empty($entity_info)) {
    return array();
  }

  if (empty($form)) {
    $form = array();
  }

  limesurvey_sync_api_entity_prepare($entity_type, $entity);

  // Basic node information.
  // These elements are just values so they are not even sent to the client.
  // Prepare datas. They may be overwritten later.
  foreach ($entity_info['entity keys'] as $generic_key => $this_entity_key) {
    if (!empty($this_entity_key)) {
      $form[$this_entity_key] = array(
        '#type' => 'value',
        '#value' => isset($entity->$this_entity_key) ? $entity->$this_entity_key : NULL,
      );
    }
  }

  if (in_array('created', $entity_info['schema_fields_sql']['base table'])) {
    // Changed must be sent to the client, for later overwrite error checking.
    $form['changed'] = array(
      '#type' => 'hidden',
      '#default_value' => isset($entity->changed) ? $entity->changed : NULL,
    );
  }

  // If the node type has a title, and the node type form defined no special
  // weight for it, we default to a weight of -5 for consistency.
  if (!empty($entity_info['entity keys']['label'])) {
    $title_field = $entity_info['entity keys']['label'];
    if (isset($form[$title_field]) && !isset($form[$title_field]['#weight'])) {
      $form[$title_field]['#weight'] = -5;
    }
  }

  if (in_array('language', $entity_info['schema_fields_sql']['base table'])) {
    // Set the Drupal language
    global $language;
    $default_dp_language = (!empty($entity->language)) ? $entity->language : ((!empty($language->language)) ? $language->language : LANGUAGE_NONE);
    if (!drupal_multilingual() || !module_exists('locale')) {
      // Only one language, don't display it.
      $default = language_default();
      $form['language'] = array(
        '#type' => 'value',
        '#value' => $default_dp_language,
      );
    }
    else {
      $languages = language_list('enabled');
      $languages = $languages[1];
      $names = array();
      foreach ($languages as $langcode => $item) {
        $names[$langcode] = t($item->name);
      }
      $form['language'] = array(
        '#type' => 'select',
        '#title' => t('Select a displayed language'),
        '#options' => $names,
        '#default_value' => $default_dp_language,
      );
    }
  }

  $form['additional_settings'] = array(
    '#type' => 'vertical_tabs',
    '#weight' => 99,
  );

  // Add a log field if the "Create new revision" option is checked, or if the
  // current user has the ability to check that option.
  if (!empty($entity_info['entity keys']['revision'])) {
    if (!isset($entity->revision)) {
      // @TODO : create an admin interface like node_type_form().
      $entity->revision = 1;
    }
    $form['revision_information'] = array(
      '#type' => 'fieldset',
      '#title' => t('Revision information'),
      '#collapsible' => TRUE,
      // Collapsed by default when "Create new revision" is unchecked
      '#collapsed' => !$entity->revision,
      '#group' => 'additional_settings',
      '#attributes' => array(
        'class' => array('node-form-revision-information'),
      ),
      '#attached' => array(
        'js' => array(drupal_get_path('module', 'node') . '/node.js'),
      ),
      '#weight' => 20,
      '#access' => $entity->revision || user_access("administer {$entity_type}"),
    );
    $form['revision_information']['revision'] = array(
      '#type' => 'checkbox',
      '#title' => t('Create new revision'),
      '#default_value' => $entity->revision,
      '#access' => user_access("administer {$entity_type}"),
    );
    // Check the revision log checkbox when the log textarea is filled in.
    // This must not happen if "Create new revision" is enabled by default, since
    // the state would auto-disable the checkbox otherwise.
    if (!$entity->revision) {
      $form['revision_information']['revision']['#states'] = array(
        'checked' => array(
          'textarea[name="log"]' => array('empty' => FALSE),
        ),
      );
    }
    // Remove the log message from the original node object.
    $entity->log = NULL;
    $form['revision_information']['log'] = array(
      '#type' => 'textarea',
      '#title' => t('Revision log message'),
      '#rows' => 4,
      '#default_value' => !empty($form_state['values']['log']) ? $form_state['values']['log'] : (!empty($entity->log) ? $entity->log : ''),
      '#description' => t('Provide an explanation of the changes you are making. This will help other authors understand your motivations.'),
    );
  }

  // Node author information for administrators
  if (in_array('uid', $entity_info['schema_fields_sql']['base table'])) {
    $form['author'] = array(
      '#type' => 'fieldset',
      '#access' => user_access("administer {$entity_type}"),
      '#title' => t('Authoring information'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#group' => 'additional_settings',
      '#attributes' => array(
        'class' => array('node-form-author'),
      ),
      '#attached' => array(
        'js' => array(
          drupal_get_path('module', 'node') . '/node.js',
          array(
            'type' => 'setting',
            'data' => array('anonymous' => variable_get('anonymous', t('Anonymous'))),
          ),
        ),
      ),
      '#weight' => 90,
    );
    $author = (isset($entity->uid)) ? user_load($entity->uid) : FALSE;
    $form['author']['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Authored by'),
      '#maxlength' => 60,
      '#autocomplete_path' => 'user/autocomplete',
      '#default_value' => $author ? check_plain(format_username($author)) : '',
      '#weight' => -1,
      '#description' => t('Leave blank for %anonymous.', array('%anonymous' => variable_get('anonymous', t('Anonymous')))),
    );

    // Date information for administrators
    if (in_array('created', $entity_info['schema_fields_sql']['base table'])) {
      //if (empty($entity->created)) {
//  $entity->created = REQUEST_TIME;
      //}
      //$entity->date = format_date($entity->created, 'custom', 'Y-m-d H:i:s O');
      $form['author']['date'] = array(
        '#type' => 'textfield',
        '#title' => t('Authored on'),
        '#maxlength' => 25,
        '#description' => t('Format: %time. The date format is YYYY-MM-DD and %timezone is the time zone offset from UTC. Leave blank to use the time of form submission.', array('%time' => !empty($entity->date) ? date_format(date_create($entity->date), 'Y-m-d H:i:s O') : format_date($entity->created, 'custom', 'Y-m-d H:i:s O'), '%timezone' => !empty($entity->date) ? date_format(date_create($entity->date), 'O') : format_date($entity->created, 'custom', 'O'))),
        '#default_value' => !empty($entity->date) ? $entity->date : '',
      );
    }
  }

  $status_col = in_array('status', $entity_info['schema_fields_sql']['base table']);
  $promote_col = in_array('promote', $entity_info['schema_fields_sql']['base table']);
  $sticky_col = in_array('sticky', $entity_info['schema_fields_sql']['base table']);
  if ($status_col || $promote_col || $sticky_col) {
    // @TODO create an interface for setting below;
    if (!isset($entity->status)) {
      $entity->status = 1;
    }
    if (!isset($entity->promote)) {
      $entity->promote = 0;
    }
    if (!isset($entity->sticky)) {
      $entity->sticky = 1;
    }
    // Node options for administrators
    $form['options'] = array(
      '#type' => 'fieldset',
      '#access' => user_access("administer {$entity_type}"),
      '#title' => t('Publishing options'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#group' => 'additional_settings',
      '#attributes' => array(
        'class' => array('node-form-options'),
      ),
      '#attached' => array(
        'js' => array(drupal_get_path('module', 'node') . '/node.js'),
      ),
      '#weight' => 95,
    );
    if ($status_col) {
      $form['options']['status'] = array(
        '#type' => 'checkbox',
        '#title' => t('Published'),
        '#default_value' => $entity->status,
      );
    }
    if ($promote_col) {
      $form['options']['promote'] = array(
        '#type' => 'checkbox',
        '#title' => t('Promoted to front page'),
        '#default_value' => $entity->promote,
      );
    }
    if ($sticky_col) {
      $form['options']['sticky'] = array(
        '#type' => 'checkbox',
        '#title' => t('Sticky at top of lists'),
        '#default_value' => $entity->sticky,
      );
    }
  }

  // Add validate et submit functions.
  // Not sure have to explicit.
  $form['#validate'][] = 'limesurvey_sync_api_entity_edit_form_validate';
  $form['#submit'][] = 'limesurvey_sync_api_entity_edit_form_submit';
  return $form;
}

// Inspired from node_form_validate();
function limesurvey_sync_api_entity_edit_form_validate($form, &$form_state) {
  $entity_type = $form_state['entity_type'];
  //$orig_entity = $form_state[$entity_type];
  // From node_form_validate()
  // $form_state['node'] contains the actual entity being edited, but we must
  // not update it with form values that have not yet been validated, so we
  // create a pseudo-entity to use during validation.
  $pseudo_entity = (object) $form_state['values'];
  limesurvey_sync_api_entity_validate($pseudo_entity, $form, $form_state);
  entity_form_field_validate($entity_type, $form, $form_state);
  //form_set_error('', 'debug');
}
// Inspired from node_validate();
function limesurvey_sync_api_entity_validate($entity, $form, $form_state) {
  $entity_type = $form_state['entity_type'];
  $entity_info = entity_get_info($entity_type);
  $id_field = $entity_info['entity keys']['id'];
  if (isset($entity->date) && in_array('changed', $entity_info['schema_fields_sql']['base table'])) {
    if (isset($entity->{$id_field}) && (limesurvey_sync_api_entity_last_changed($entity_type, $entity) > $entity->date)) {
      form_set_error('date', t('The content on this page has either been modified by another user, or you have already submitted modifications using this form. As a result, your changes cannot be saved.'));
    }
  }

  if (isset($entity->name) && in_array('uid', $entity_info['schema_fields_sql']['base table'])) {
    // Validate the "authored by" field.
    $name = $entity->name;
    if (!empty($name) && !($account = user_load_by_name($name))) {
      // The use of empty() is mandatory in the context of usernames
      // as the empty string denotes the anonymous user. In case we
      // are dealing with an anonymous user we set the user ID to 0.
      form_set_error('name', t('The username %name does not exist.', array('%name' => $name)));
    }
  }

  if (isset($entity->date) && in_array('changed', $entity_info['schema_fields_sql']['base table'])) {
    // Validate the "authored on" field.
    $date = $entity->date;
    if (!empty($date) && strtotime($date) === FALSE) {
      form_set_error('date', t('You have to specify a valid date.'));
    }
  }
}

// Inspired from node_form_submit();
function limesurvey_sync_api_entity_edit_form_submit($form, &$form_state) {
  // @todo Legacy support for modules that extend the node form with form-level
//   submit handlers that adjust $form_state['values'] prior to those values
//   being used to update the entity. Module authors are encouraged to instead
//   adjust the node directly within a hook_node_submit() implementation. For
//   Drupal 8, evaluate whether the pattern of triggering form-level submit
//   handlers during button-level submit processing is worth supporting
//   properly, and if so, add a Form API function for doing so.
  // Not sure below is needed...
  //unset($form_state['submit_handlers']);
  //form_execute_handlers('submit', $form, $form_state);

  $entity_type = $form_state['entity_type'];
  $entity_info = entity_get_info($entity_type);
  $id_field = $entity_info['entity keys']['id'];
  // retreive the original entity.
  $entity = $form_state[$entity_type];
  $insert = empty($entity->{$id_field});
  // apply values form form and fields forms to $entity.
  entity_form_submit_build_entity($entity_type, $entity, $form, $form_state);
  // Delete remaining garbage datas from entity_ui_form_submit_build_entity().
  $garbage = array(
    'submit',
    'form_build_id',
    'form_token',
    'form_id',
    'op',
    'delete',
  );

  foreach ($garbage as $garbage_field) {
    if (isset($entity->$garbage_field)) {
      unset($entity->$garbage_field);
    }
  }

  // Specific changes on node (author, date).
  limesurvey_sync_api_entity_edit_submit($entity, $entity_type);

  entity_save($entity_type, $entity);

  // watchdog is handle into the controller class save method
  // Display message.
  // retreive type name
  $bundle_parameter_needed = !empty($entity_info['entity keys']['bundle']);
  $type_name = ($bundle_parameter_needed) ? $entity_info['bundles'][$entity->{$entity_info['entity keys']['bundle']}]['label'] : $entity_info['label'];

  $t_args = array(
    '@type' => $type_name,
    '%title' => entity_label($entity_type, $entity),
  );
  if ($insert) {
    drupal_set_message(t('@type %title has been created.', $t_args));
  }
  else {
    drupal_set_message(t('@type %title has been updated.', $t_args));
  }
  if ($entity->{$id_field}) {
    $form_state['values'][$id_field] = $entity->{$id_field};
    $form_state[$id_field] = $entity->{$id_field};
    $form_state['redirect'] = entity_access('view', $entity_type, $entity) ? current(entity_uri($entity_type, $entity)) : '<front>';
  }
  else {
    // In the unlikely case something went wrong on save, the node will be
    // rebuilt and node form redisplayed the same way as in preview.
    drupal_set_message(t('The data could not be saved.'), 'error');
    $form_state['rebuild'] = TRUE;
  }
  // Clear the page and block caches.
  cache_clear_all();
}

// Inspired from node_submit();
function limesurvey_sync_api_entity_edit_submit($entity, $entity_type = NULL) {
  $entity_type = (!empty($entity_type)) ? $entity_type : $entity->entity_type;
  $entity_info = entity_get_info($entity_type);

  if (in_array('uid', $entity_info['schema_fields_sql']['base table'])) {
    // A user might assign the node author by entering a user name in the node
    // form, which we then need to translate to a user ID.
    if (isset($entity->name)) {
      if ($account = user_load_by_name($entity->name)) {
        $entity->uid = $account->uid;
      }
      else {
        $entity->uid = 0;
      }
    }
  }

  if (in_array('created', $entity_info['schema_fields_sql']['base table'])) {
    $entity->created = !empty($entity->date) ? strtotime($entity->date) : REQUEST_TIME;
    $entity->validated = TRUE;
  }
  return $entity;
}

// Button delete redirect to delete page
function limesurvey_sync_api_entity_delete_form_submit($form, &$form_state) {
  $entity_type = $form_state['entity_type'];
  $entity = $form_state[$entity_type];
  $entity_path = current(entity_uri($entity_type, $entity));
  drupal_goto($entity_path . '/delete');
}

// Fork of the entity_ui_get_form() function, add possibiity to add entity with bundle and change the wrapper for adding buton and options, as author, etc...
// Called from class:hook_menu()
function limesurvey_sync_api_entity_entity_ui_get_form($entity_type, $entity, $op = 'edit', $form_state = array(), $bundle_provided = NULL) {

  $entity_info = entity_get_info($entity_type);
  if (isset($entity)) {
    list(, , $bundle) = entity_extract_ids($entity_type, $entity);
  }

  $form_id = (!isset($bundle) || $bundle == $entity_type) ? $entity_type . '_form' : $entity_type . '_edit_' . $bundle . '_form';


  // Here need to create the entity. The bundle may be required.
  // Try to guess it or use the one provided.
  if (!isset($entity) && $op == 'add') {
    // Handle the bundle parameter.
    $bundle_parameter_needed = !empty($entity_info['entity keys']['bundle']);
    $only_1_bundle = ($bundle_parameter_needed && count($entity_info['bundles']) == 1 && !empty($entity_info['bundles'])) ? key($entity_info['bundles']) : FALSE;
    // Find the Bundle entity type.
    $bundle_entity_type = limesurvey_sync_api_entity_find_bundle_entity_type($entity_type);
    $only_1_bundle_valid = ($only_1_bundle && !empty(entity_load_multiple_by_name($bundle_entity_type, array($only_1_bundle))));
    $many_bundles = ($bundle_parameter_needed && count($entity_info['bundles']) > 1);
    // No act depending on the situation.
    $values = array();
    if ($only_1_bundle) {
      if ($only_1_bundle_valid) {
        $values = array($entity_info['entity keys']['bundle'] => $only_1_bundle);
      }
      else {
        drupal_not_found();
        exit;
      }
    }
    elseif ($many_bundles) {
      if (!empty($bundle_provided) && !empty(entity_load_multiple_by_name($bundle_entity_type, array($bundle_provided)))) {
        $values = array($entity_info['entity keys']['bundle'] => $bundle_provided);

      }
      else {
        drupal_not_found();
        exit;
      }
    }

    $entity = entity_create($entity_type, $values);
  }

  // Do not use drupal_get_form(), but invoke drupal_build_form() ourself so
  // we can prepulate the form state.
  //$form_state['wrapper_callback'] = 'entity_ui_main_form_defaults';
  // HERE : I change for my wrapper callback
  $form_state['wrapper_callback'] = 'limesurvey_sync_api_entity_entity_ui_main_form_defaults';
  $form_state['entity_type'] = $entity_type;
  form_load_include($form_state, 'inc', 'entity', 'includes/entity.ui');

  // Handle cloning. We cannot do that in the wrapper callback as it is too late
  // for changing arguments.
  if ($op == 'clone') {
    $entity = entity_ui_clone_entity($entity_type, $entity);
    // Add ' cloned' to the title.
    if (!empty($entity_info['entity keys']['label'])) {
      $entity->{$entity_info['entity keys']['label']} .= ' (cloned)';
    }
  }

  // We don't pass the entity type as first parameter, as the implementing
  // module knows the type anyway. However, in order to allow for efficient
  // hook_forms() implementiations we append the entity type as last argument,
  // which the module implementing the form constructor may safely ignore.
  // @see entity_forms()
  $form_state['build_info']['args'] = array($entity, $op, $entity_type);
  return drupal_build_form($form_id, $form_state);
}

// Rewrite entity_ui_main_form_defaults() adding more elements
function limesurvey_sync_api_entity_entity_ui_main_form_defaults($form, &$form_state, $entity = NULL, $op = NULL) {
  // Now equals entity_ui_form_defaults() but is still here to keep backward
  // compatibility.
  $form = entity_ui_form_defaults($form, $form_state, $form_state['entity_type'], $entity, $op);
  // The entity API has added its stuf, my turn.
  // Delete entity_ui validatation and submit fonctions from entity_ui_form_defaults()
  //unset($form['#validate'], $form['#submit']);

  $entity_type = $form_state['entity_type'];
  // Prepare entity regarding revison, author, published

  // add stuff for revison, author, published,
  limesurvey_sync_api_entity_edit_form($form, $form_state, $entity, $entity_type);

  // Load the entity_reference field (and maybe more custon fields).
  field_attach_form($entity_type, $entity, $form, $form_state);

  // Add buttons
  $op = $form_state['op'];
  $form['actions'] = array(
    '#type' => 'actions',
  );
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => empty($entity->is_new) ? t('Update') : t('Save'),
    '#weight' => 40,
  );

  if ($op != 'add' && $op != 'clone') { // !$entity->isLocked() &&
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#weight' => 45,
      '#limit_validation_errors' => array(),
      '#submit' => array('limesurvey_sync_api_entity_delete_form_submit'),
      '#access' => entity_access('delete', $entity_type, $entity),
    );
  }

  // Add validations and submit
  $form_id = $form_state['build_info']['form_id'];
  foreach (array('validate', 'submit') as $item) {
    $item_id = $form_id . '_' . $item;
    if (function_exists($item_id)) {
      // execute before entity_save
      array_unshift($form['#' . $item], $item_id);
    }
  }

  return $form;
}

// Return the Bundle entity_type name (str) for an entity_type (str) or False if not defined.
function limesurvey_sync_api_entity_find_bundle_entity_type($entity_type) {
  if (empty($entity_type)) {
    return FALSE;
  }
  $bundle_entity_type = FALSE;
  foreach (entity_get_info() as $an_entity_type => $datas) {
    if (!empty($datas['bundle of']) && $datas['bundle of'] == $entity_type) {
      $bundle_entity_type = $an_entity_type;
      break;
    }
  }
  return $bundle_entity_type;
}

/**
 * Finds the last time an entity was changed.
 * From node_last_changed().
 *
 * @param $nid
 *   The ID of a node.
 *
 * @return
 *   A unix timestamp indicating the last time the entity was changed or FALSE.
 */
function limesurvey_sync_api_entity_last_changed($entity_type, $entity) {
  $entity_info = entity_get_info($entity_type);
  $col_changed = 'changed';
  if ($entity_info && is_object($entity) && in_array($col_changed, $entity_info['schema_fields_sql']['base table']) && !empty($entity_info['entity keys']['id'])) {
    $id_field = $entity_info['entity keys']['id'];
    $id = $entity->{$id_field};
    if (!empty($id)) {
      return db_query('SELECT ' . $col_changed . ' FROM {' . $entity_info['base table'] . '} WHERE ' . $id_field . ' = :id', array(':id' => $id))->fetch()->{$col_changed};
    }
  }
  return FALSE;
}

// Fork of the node_object_prepare($node) core function
function limesurvey_sync_api_entity_prepare($entity_type, $entity) {

  $entity_info = entity_get_info($entity_type);
  // Set up default values, if required.
  $bundle = (!empty($entity_info['entity keys']['bundle'])) ? $entity->{$entity_info['entity keys']['bundle']} : $entity_type;
  $esan_options = variable_get('esan_options_' . $bundle, array('status', 'promote'));
  // If this is a new node, fill in the default values.
  if (!isset($entity->{$entity_info['entity keys']['id']}) || isset($entity->is_new)) {
    foreach (array('status', 'promote', 'sticky') as $key) {
      // Multistep node forms might have filled in something already.
      if (!isset($entity->$key) && isset($entity_info['schema_fields_sql']['base table'][$key])) {
        $entity->$key = (int) in_array($key, $esan_options);
      }
    }
    if (in_array('uid', $entity_info['schema_fields_sql']['base table'])) {
      global $user;
      $entity->uid = $user->uid;
    }
    if (in_array('created', $entity_info['schema_fields_sql']['base table'])) {
      $entity->created = REQUEST_TIME;
    }
  }
  else {
    if (in_array('created', $entity_info['schema_fields_sql']['base table'])) {
      $entity->date = format_date($entity->created, 'custom', 'Y-m-d H:i:s O');
    }
    if (in_array('log', $entity_info['schema_fields_sql']['base table'])) {
      // Remove the log message from the original node object.
      $entity->log = NULL;
    }
  }

  if (!empty($entity_info['entity keys']['revision'])) {
    // Always use the default revision setting.
    $entity->revision = in_array('revision', $esan_options);
  }
}
