<?php
/**
 * @file
 * Rules specific datas the LimeSurvey Sync module.
 */

/**
 * Implements hook_rules_condition_info().
 *
 */
function limesurvey_sync_rules_condition_info() {
  $conditions = array();
  $default = array(
    'group' => t('LimeSurvey Sync'),
    'module' => 'limesurvey_sync',
    'parameter' => array(
      'entity' => array(
        'type' => 'entity',
        'label' => t('The LimeSurvey entity'),
        'description' => t('Specifies the entity for which to evaluate the condition.'),
      ),
    ),
  );
  $conditions['limesurvey_sync_rules_condition_is_synchronized'] = $default + array(
      'label' => t('LimeSurvey entity is synchronized'),
    );
  $conditions['limesurvey_sync_rules_condition_is_ls_entity'] = $default + array(
      'label' => t('Entity is a LimeSurvey entity'),
    );
  $conditions['limesurvey_sync_rules_condition_is_type_of'] = $default + array(
      'label' => t('LimeSurvey entity is type of'),
    );
  $conditions['limesurvey_sync_rules_condition_is_type_of']['parameter']['ls_entity_type'] = array(
    'type' => 'token',
    'label' => t('LimeSurvey entity type'),
    'description' => t('The type of the checked LimeSurvey entity.'),
    'options list' => 'limesurvey_sync_rules_ls_entity_type_options',
    'restriction' => 'input',
  );

  return $conditions;
}

/**
 * return TRUE if an answer is synchronized.
 */
function limesurvey_sync_rules_condition_is_synchronized($entity) {
  // $entity is provided trought the entity_metada_wrapper. we fix it.
  if (!limesurvey_sync_rules_condition_is_ls_entity($entity)) {
    // This is not a LimeSurvey entity.
    return FALSE;
  }
  if ($entity instanceof EntityDrupalWrapper) {
    $entity = $entity->value();
  }
  $synchronize_set = $entity->ls_synchronize();
  return (isset($synchronize_set['ok']));
}

/**
 * return TRUE if the entity is a LimeSurvey entity
 */
function limesurvey_sync_rules_condition_is_ls_entity($entity) {
  if (!limesurvey_sync_rules_condition_is_ls_entity($entity)) {
    // This is not a LimeSurvey entity.
    return FALSE;
  }
  // $entity is provided trought the entity_metada_wrapper. we fix it.
  if ($entity instanceof EntityDrupalWrapper) {
    $entity = $entity->value();
  }
  return (isset($entity->ls_entity_type));
}

/**
 * return TRUE if the entity is a type of the provided type.
 *
 * Allowed types are 'answer', 'token', 'response', 'survey'.
 */
function limesurvey_sync_rules_condition_is_type_of($entity, $ls_entity_type) {
  if (!limesurvey_sync_rules_condition_is_ls_entity($entity)) {
    // This is not a LimeSurvey entity.
    return FALSE;
  }
  // $entity is provided trought the entity_metada_wrapper. we fix it.
  if ($entity instanceof EntityDrupalWrapper) {
    $entity = $entity->value();
  }
  $result = (isset($entity->ls_entity_type) && $entity->ls_entity_type == $ls_entity_type);
  return (isset($entity->ls_entity_type) && $entity->ls_entity_type == $ls_entity_type);
}

function limesurvey_sync_rules_ls_entity_type_options() {
  return array(
    'answer' => t('Answer'),
    'survey' => t('Survey'),
    'response' => t('Response (set of an answer and its associated LimeSurvey token)'),
    'token' => t('LimeSurvey token'),
  );
}

/**
 * Implements hook_rules_action_info().
 *
 */
function limesurvey_sync_rules_action_info() {
  $actions = array();
  $default = array(
    'group' => t('LimeSurvey Sync'),
    'module' => 'limesurvey_sync',
    'parameter' => array(
      'entity' => array(
        'type' => 'entity',
        'label' => t('The LimeSurvey entity'),
        'description' => t('Specifies the LimeSurvey entity for which the action will be fired.'),
      ),
    ),
  );

  $actions['limesurvey_sync_rules_action_synchronize'] = $default + array(
      'label' => t('Synchronize the LimeSurvey entity'),
    );
  $actions['limesurvey_sync_rules_action_synchronize']['parameter']['way'] = array(
    'type' => 'token',
    'label' => t('Synchronization direction'),
    'description' => t('If datas are different between Drupal and LimeSurvey, choose the one which will overwrite the other'),
    'options list' => 'limesurvey_sync_rules_synchronize_way_options',
    'default value' => 'ls2dp',
    'restriction' => 'input',
  );

  return $actions;
}

function limesurvey_sync_rules_action_synchronize($entity, $way) {
  if (!limesurvey_sync_rules_condition_is_ls_entity($entity)) {
    // This is not a LimeSurvey entity.
    return FALSE;
  }
  // $entity is provided trought entity_metada_wrapper().
  if ($way == 'ls2dp') {
    // Flag not needed, the synchronization will occure only if needed.
    //$entity->need_save_ls2dp = TRUE;
  }
  elseif ($way == 'dp2ls') {
    $entity->need_save_dp2ls->set(TRUE);
  }
  $entity->save();
}

function limesurvey_sync_rules_synchronize_way_options() {
  return array(
    'ls2dp' => t('Synchronize from LimeSurvey to Drupal (recommanded, by default)'),
    'dp2ls' => t('Synchronize from Drupal to LimeSurvey (experimental)'),
  );
}